/**
 * MainContent component
 * Dynamically fetches and renders the content for the main page (homepage)
 * using blocks defined in the admin editor.
 */
import React, { useState, useEffect, useMemo } from 'react';
import { ContentBlock } from './blocks/BlockRenderer'; // Import types
import { renderOptimizedBlock } from './blocks/OptimizedBlockRenderer'; // Import optimized renderer
import './MainContent.css'; // Keep existing styles if they are general enough

// Interface for the data expected from /api/pages/main
// This is similar to CaseData, ensure it matches the API response for the main page.
interface MainPageData {
  identifier: string; // Should be 'main'
  title: string;
  overview: ContentBlock[] | null; // Array of content blocks
  updatedAt?: string; // Optional
  // Add any other fields your main page API might return
}

// Fallback data for immediate rendering while API loads
const FALLBACK_DATA: MainPageData = {
  identifier: 'main',
  title: 'Studio Paulides',
  overview: [
    {
      id: 'heading-main',
      type: 'heading',
      content: 'Geweldig dat je hier bent'
    },
    {
      id: 'text-intro',
      type: 'text',
      content: '<p>Ik ben Folkert Paulides, een designer gespecialiseerd in digitale productontwikkeling. Samen met andere specialisten ga ik graag de uitdaging aan om de beste digitale gebruikerservaringen te creëren. Ik werk op freelance basis voor organisaties in verschillende sectoren.</p>'
    },
    {
      id: 'subheading-why',
      type: 'subheading',
      content: 'Waarom?'
    },
    {
      id: 'text-why',
      type: 'text',
      content: '<p>Digitalisering neemt een steeds prominentere plek in ons dagelijks leven in. Technologie en digitalisering zijn niet meer weg te denken uit de workflow van organisaties. Digitaal is de norm.</p>'
    },
    {
      id: 'contact-section',
      type: 'contact'
    }
  ]
};

const MainContent: React.FC = () => {
  const [pageData, setPageData] = useState<MainPageData>(FALLBACK_DATA); // Start with fallback data
  const [isLoading, setIsLoading] = useState<boolean>(false); // Start as false since we have fallback
  const [isUsingFallback, setIsUsingFallback] = useState<boolean>(true);

  useEffect(() => {
    const fetchMainPageData = async () => {
      setIsLoading(true);
      console.log('MainContent: Fetching fresh data for main page...');

      try {
        const controller = new AbortController();
        const timeoutSignal = setTimeout(() => controller.abort(), 5000); // Reduced timeout to 5 seconds

        const response = await fetch('/api/pages/main', {
          signal: controller.signal,
          // Let Vercel and Service Worker handle caching properly
        });
        
        clearTimeout(timeoutSignal);

        if (!response.ok) {
          const errorData = await response.json().catch(() => ({}));
          const errorMessage = errorData.message || `Failed to fetch main page data. Status: ${response.status}`;
          console.warn('MainContent: API Error, using fallback data -', errorMessage);
          // Don't throw error, just log and continue with fallback
          return;
        }

        const data: MainPageData = await response.json();
        console.log('MainContent: Fresh data fetched successfully', data);

        // Only update if we actually got different data
        if (JSON.stringify(data.overview) !== JSON.stringify(FALLBACK_DATA.overview)) {
          setPageData({
            ...data,
            overview: Array.isArray(data.overview) ? data.overview : FALLBACK_DATA.overview
          });
          setIsUsingFallback(false);
        }
        
        document.title = `${data.title || 'Homepage'} - Studio Paulides`;

      } catch (err: any) {
        console.warn("MainContent: Error fetching fresh data, using fallback:", err.message);
        // Don't set error state, just continue with fallback data
        if (err.name === 'AbortError') {
          console.warn('MainContent: Request timeout, using fallback data');
        }
      } finally {
        setIsLoading(false);
      }
    };

    // Small delay to allow initial render with fallback data
    const timeoutId = setTimeout(fetchMainPageData, 100);
    return () => clearTimeout(timeoutId);
  }, []); // Empty dependency array ensures this runs once on mount

  // Show loading skeleton only if we don't have any data at all
  if (!pageData || !pageData.overview) {
    return (
      <main className="main-content-area">
        <article 
          className="content-section" 
          style={{ 
            paddingTop: 'var(--spacing-5)',
            paddingLeft: '1rem', 
            paddingRight: '1rem' 
          }} 
        >
          {/* Simplified skeleton loader */}
          <div className="skeleton-loader">
            <div className="skeleton-heading" style={{
              height: '3rem',
              backgroundColor: 'var(--color-z-30)',
              borderRadius: '8px',
              marginBottom: '1.5rem',
              width: '70%',
              animation: 'pulse 1.5s ease-in-out infinite alternate'
            }}></div>
            
            <div style={{
              height: '1rem',
              backgroundColor: 'var(--color-z-30)',
              borderRadius: '4px',
              marginBottom: '0.5rem',
              animation: 'pulse 1.5s ease-in-out infinite alternate'
            }}></div>
          </div>

          <style>{`
            @keyframes pulse {
              0% { opacity: 0.6; }
              100% { opacity: 1; }
            }
          `}</style>
        </article>
      </main>
    );
  }

  // Memoize expensive computations to prevent re-calculations on every render
  const contentAnalysis = useMemo(() => {
    if (!pageData.overview) {
      return { hasHeadingBlock: false, hasH1InContent: false, needsMainHeading: true };
    }
    
    const hasHeadingBlock = pageData.overview.some(block => block.type === 'heading');
    const firstTextBlock = pageData.overview.find(block => block.type === 'text') as any;
    const hasH1InContent = firstTextBlock && firstTextBlock.content?.includes('<h1');
    return {
      hasHeadingBlock,
      hasH1InContent,
      needsMainHeading: !hasHeadingBlock && !hasH1InContent
    };
  }, [pageData.overview]);

  // Memoize rendered blocks to prevent unnecessary re-renders
  const renderedBlocks = useMemo(() => {
    if (!pageData.overview) return [];
    return pageData.overview.map(block => renderOptimizedBlock(block));
  }, [pageData.overview]);

  return (
    <main className="main-content-area">
      <article 
        className="content-section" 
        style={{ 
          paddingTop: 'var(--spacing-5)',
          paddingLeft: '1rem', 
          paddingRight: '1rem',
          animation: 'fadeIn 0.3s ease-in-out', // Faster animation
          opacity: isLoading ? 0.8 : 1, // Subtle loading indicator
          transition: 'opacity 0.2s ease-in-out'
        }} 
      >
        {/* Add main heading if not present in content blocks */}
        {contentAnalysis.needsMainHeading && (
          <h1 className="main-heading" style={{ marginBottom: '1.5rem' }}>
            Geweldig dat je hier bent
          </h1>
        )}
        
        {/* Render the optimized blocks */}
        {renderedBlocks}
        
        {/* Show subtle indicator if using fallback data */}
        {isUsingFallback && process.env.NODE_ENV === 'development' && (
          <div style={{
            position: 'fixed',
            bottom: '1rem',
            right: '1rem',
            background: 'rgba(255, 165, 0, 0.1)',
            border: '1px solid orange',
            borderRadius: '4px',
            padding: '0.5rem',
            fontSize: '0.75rem',
            color: 'orange',
            zIndex: 1000
          }}>
            Using fallback data
          </div>
        )}
        
        {/* Add CSS for fade-in animation */}
        <style>{`
          @keyframes fadeIn {
            0% { 
              opacity: 0; 
              transform: translateY(10px);
            }
            100% { 
              opacity: 1; 
              transform: translateY(0);
            }
          }
        `}</style>
      </article>
    </main>
  );
};

export default MainContent; 